/*
 *		Virtual Book Applet
 *				1997/04/23 (C) Copyright T.Kazawa
 */


import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.applet.*;
import java.util.*;

class BookCanvas extends Canvas
{
	Image image = null;
	
	public void paint(Graphics g) {
		Dimension canvassize = this.size();
		g.clearRect(0, 0, canvassize.width, canvassize.height);
		g.clipRect(0, 0, canvassize.width, canvassize.height);
		if (image == null) {
			g.setFont(new Font("TimesRoman", Font.BOLD, 16));
			g.setColor(Color.red);
			g.drawString("Now loading ... Please wait.", 8, 16);
			return;
		}
		g.drawImage(image
			, (canvassize.width-image.getWidth(this)) / 2
			, (canvassize.height-image.getHeight(this)) / 2
			, this);
	}
	
	public void setImage(Image img) {
		image = img;
	}
}

public class Book extends Applet implements Runnable {
	int allpages, currentpage, currentloadingimage;
	String filenamebase, subframe;
	Image[] imgbuf;
	boolean[] imgflag;
	Thread thread = null;
	MediaTracker tracker;
	Button prev, next;
	Label pages;
	BookCanvas canvas;
	
	/*
	 * Initialize the applet. Resize and load images.
	 */
	public void init()
	{
		currentpage = 1;
		
		/* Get parameter */
		allpages = Integer.parseInt((getParameter("pages") != null)
			? getParameter("pages") : "0");
		filenamebase = getParameter("filename");
		subframe = getParameter("subframe");
		
		/* Image data load setup */
		imgbuf = new Image[allpages+1];
		imgflag = new boolean[allpages+1];
		tracker = new MediaTracker(this);
		currentloadingimage = 1;
		
		/* Applet Setup */
		setLayout(new BorderLayout());
		canvas = new BookCanvas();
		canvas.setBackground(Color.white);
		add("Center", canvas);
		prev = new Button("Prev");
		next = new Button("Next");
		pages = new Label();
		pages.setAlignment(Label.CENTER);
		add("West", next);
		add("South", pages);
		add("East", prev);
		
		/* Get focus */
		next.requestFocus();
	}
	
	public boolean action(Event e, Object arg) {
		if (e.target == next) {
			currentpage++;
			buttonChange();
			repaint();
			return true;
		} else if (e.target == prev) {
			currentpage--;
			buttonChange();
			repaint();
			return true;
		} else
			return false;
	}
	
	void buttonChange() {
		if (currentpage + 1 > allpages
			|| imgflag[currentpage+1] == false) {
			next.disable();
		} else {
			next.enable();
		}
		if (currentpage == 1
			|| imgflag[currentpage-1] == false) {
			prev.disable();
		} else {
			prev.enable();
		}
	}
	
	public void repaint() {
		super.repaint();
		String subhtml = getParameter(Integer.toString(currentpage));
		if (subframe != null && subhtml != null)
			try {
				getAppletContext().showDocument(new URL(getDocumentBase()
					, subhtml), subframe);
			} catch (MalformedURLException ex) {
				ex.printStackTrace();
			}
	}
	
	public void paint(Graphics g) {
		if (imgflag[currentpage] == true)
			canvas.setImage(imgbuf[currentpage]);
		canvas.repaint();
		pages.setText(currentpage+"/"+allpages);
	}
	
	/*
	 *		Thread routine
	 */
	public void start()
	{
		buttonChange();
		repaint();
		if (thread == null) {
			thread = new Thread(this);
			thread.start();
		}
	}
	
	public void stop()
	{
		thread = null;
	}
	
	public void run()
	{
		while (thread != null && currentloadingimage <= allpages) {
			StringTokenizer tokenizer = new StringTokenizer(filenamebase, ".");
			String filename = tokenizer.nextToken()+currentloadingimage
				+"."+tokenizer.nextToken();
			showStatus("Loading ... "+filename);
			
			imgbuf[currentloadingimage]=getImage(getDocumentBase(), filename);
			tracker.addImage(imgbuf[currentloadingimage], currentloadingimage);
			try {
				tracker.waitForID(currentloadingimage);
			} catch (InterruptedException e) {
				continue;
			}
			imgflag[currentloadingimage] = true;
			if (currentloadingimage == currentpage)
				repaint();
			currentloadingimage++;
			buttonChange();
//			thread.yield();
			try {
				thread.sleep(100);
			} catch (InterruptedException ex) {}
		}
		showStatus("Complete.");
		thread = null;
	}
	
	public String getAppletInfo()
	{
		return "Virtual Book Applet by T.Kazawa (C) 1997/04/23";
	}

//		public void destroy()
//		{
//		}
}
